; ***********************************************
; *** KDemo - 21-Apr-2007 by Kevin Krausnick  ***
; *** Ported to Acme Assembler on 14-Sep-2024 ***
; ***********************************************

* = 2049

;CREDITS - "10 SYS2082:REM BY KEVIN KRAUSNICK"
!BYTE 32,8,10,0,158,50,48,56,50,58,143,32,66,89,32,75
!BYTE 69,86,73,78,32,75,82,65,85,83,78,73,67,75,0,0,0

JSR INITSID
JSR SETUPSCR
MAINLOOP = *
JSR SETDELAY
JSR FLIPFRAME
JSR ANIMATE
JSR MOVEBEE
JSR WAITDELAY
JSR $FFE4
BEQ MAINLOOP
JSR RESTORESCR
RTS

SETUPSCR = *

LDA $D011 ;BLANK THE SCREEN
AND #$EF
STA $D011

; *** ENTER BIT MAP MODE ***
LDA #0 ;BLACK
STA 53280 ;BORDER COLOR
STA 53281 ;BACKGROUND COLOR
LDA #132 ;32768+1024=33792
STA 648 ;MOVE SCREEN
LDA 56576
AND #252
ORA #1
STA 56576 ;ENABLE BANK 2
LDA 53272
ORA #8
STA 53272 ;BIT MAP OFFSET 8K
LDA 53265
ORA #32
STA 53265 ;ENABLE BIT MAP MODE

; *** SELECT SCREEN COLOR ***
GETCOLOR = *
LDY #0
GC1 = *
JSR RANDOMIZE
AND #$F
CMP COLORS,Y
BEQ FOUNDCOLOR
INY
CPY #4
BNE GC1
JMP GETCOLOR
FOUNDCOLOR = *
ROL
ROL
ROL
ROL
AND #$F0

; *** SET SCREEN COLOR ***
LDY #0
NEXTBLOCK = *
STA $8400,Y
STA $8500,Y
STA $8600,Y
STA $8700,Y
DEY
BNE NEXTBLOCK

; *** SELECT TILING PATTERN ***
RP1 = *
JSR RANDOMIZE
CMP #15
BCS RP1
ASL
ASL
ASL
STA PATSTART
ADC #8
STA PATEND

; *** RENDER TILING ON BACKGROUND
LDA #$A0
STA $BC
LDA #0
STA $BB
TAY
LDX PATSTART
NEXTTILE LDA PATTERNS,X
STA ($BB),Y
INY
INX
CPX PATEND
BNE NEXTTILE
LDX PATSTART
CPY #0
BNE NEXTTILE
INC $BC
LDA $BC
CMP #$C0
BNE NEXTTILE

; *** ACTIVATE SPRITES ***
SPRINIT = *
LDA #125
STA 53248 ;SPRITE 1 X
LDA #140
STA 53249 ;SPRITE 1 Y
STA 53253 ;SPRITE 3 Y
LDA #215
STA 53252 ;SPRITE 3 X
LDA #0
STA 53250 ;SPRITE 2 X
AS1 = *
JSR RANDOMIZE
CMP #$34
BCC AS1
CMP #$E0
BCS AS1
STA 53251 ;SPRITE 2 Y
LDA #255
STA 53276 ;MULTICOLOR MODE
LDA #11
STA 53285 ;MULTICOLOR REG 0
LDA #15
STA 53286 ;MULTICOLOR REG 1
LDA #1
STA 53288 ;MULTICOLOR SPR 1
LDA #10
STA 53289 ;FLESH TONE FACE
LDA #7
STA 53269 ;ENABLE SPRITES 1-3

LDA $D011 ;UNBLANK THE SCREEN
ORA #$10
STA $D011
RTS

; *** SPRITE POSITION REGS ***
ANIMATE = *
LDA #$D0 ;HIGH BYTE X/Y POSITION
STA $BC
STA $BE
LDX #0
STX $BB ;LOW BYTE X POSITION
INX
STX $BD ;LOW BYTE Y POSITION

; *** MOVE SPRITES ***
LDA #1
STA SPRITENO
LDY #0
JSR MOVEY ;MOVE VERTICALLY
JSR MOVEX ;MOVE HORIZONTALLY
LDA #4
STA SPRITENO
TAY
JSR MOVEY ;MOVE VERTICALLY
JSR MOVEX ;MOVE HORIZONTALLY
RTS ;RETURN TO BASIC

; *** MOVE VERTICALLY ***
MOVEY = *
JSR RANDOMIZE ;UP/DN FLAG
BIT SPRITENO
BNE MOVEUP
LDA ($BD),Y
CLC
ADC #1
STA ($BD),Y ;DOWN 1 PIXEL
BCC MVDN1
LDA #$1F
STA ($BD),Y
MVDN1 = *
JMP CHECKYEND

MOVEUP = *
LDA ($BD),Y
SEC
SBC #1
CMP #$1E
BCS SAVEDN
LDA #$FF
SAVEDN = *
STA ($BD),Y ;UP 1 PIXEL
CHECKYEND = *
RTS

; *** MOVE HORIZONTALLY ***
MOVEX = *
JSR RANDOMIZE ;LEFT/RIGHT FLAG
BIT SPRITENO
BNE MOVELEFT

; --- PREPARE TO MOVE RIGHT ---
LDA ($BB),Y
CLC
ADC #1
STA ($BB),Y
BNE CHECKX1
LDA $D010
ORA SPRITENO
STA $D010
JMP CHECKXEND

CHECKX1 = *
LDA SPRITENO
BIT $D010
BEQ CHECKXEND
LDA ($BB),Y
CMP #$58
BCC CHECKXEND
LDA #0
STA ($BB),Y
LDA SPRITENO
EOR #$FF
STA TEMP1
LDA $D010
AND TEMP1
STA $D010
JMP CHECKXEND

; --- PREPARE TO MOVE LEFT ---
MOVELEFT = *
LDA ($BB),Y
SEC
SBC #1
STA ($BB),Y
BCS CHECKXEND
LDA $D010
BIT SPRITENO
BNE CHECKX2
LDA $D010
ORA SPRITENO
STA $D010
LDA #$57
STA ($BB),Y
JMP CHECKXEND

CHECKX2 = *
LDA SPRITENO
EOR #$FF
STA TEMP1
LDA $D010
AND TEMP1
STA $D010
LDA #$FF
STA ($BB),Y
CHECKXEND = *
RTS

; *** MOVE BEE RIGHT OR LEFT ***

MOVEBEE = *
LDA BDIRECTION
AND #1
BNE BLEFT

; --- MOVE BEE RIGHT ---
LDA $D002
CLC
ADC #3
STA $D002
BCC BR1
LDA $D010
ORA #2
STA $D010
LDA #0
STA $D002
JMP BREND

BR1 LDA #2
BIT $D010
BEQ BREND
LDA $D002
CMP #88
BCC BREND
GETNEWY JSR RANDOMIZE
CMP #150
BCS GETNEWY
ADC #50
STA $D003
LDA BDIRECTION
EOR #1
STA BDIRECTION
BREND RTS

; --- MOVE BEE LEFT ---

BLEFT = *
LDA $D002
SEC
SBC #3
STA $D002
BCS BLEND

LDA #2
BIT $D010
BEQ BL1
LDA $D010
AND #$FD
STA $D010
LDA #$FF
STA $D002
JMP BLEND

BL1 LDA #1
STA $D002
LDA BDIRECTION
EOR #1
STA BDIRECTION
BLEND RTS

; *** FLIP THROUGH SPRITE FRAMES

FLIPFRAME = *
INC RFRAME
LDA RFRAME
CMP #12
BNE FF1
LDA #0
STA RFRAME
FF1 LDA RFRAME
CLC
ADC #32
STA 34808

INC FFRAME
LDA FFRAME
CMP #6
BNE FF2
LDA #0
STA FFRAME
FF2 LDA FFRAME
CLC
LSR
CLC
ADC #44
STA 34810

INC BFRAME
LDA BFRAME
CMP #2
BNE FF3
LDA #0
STA BFRAME
FF3 LDA #1
BIT BDIRECTION
BEQ FF4
LDA BFRAME
CLC
ADC #49
JMP FFEND

FF4 LDA BFRAME
CLC
ADC #47
FFEND STA 34809
RTS

; *** USE SID AS RANDOMIZER ***
INITSID = *
LDA #$FF
STA $D40F ;VOICE 3 FREQ HIGH BYTE
LDA #$F0
STA $D414 ;VOICE 3 SUSTAIN VAL
LDA #$80
STA $D412 ;VOICE 3 WAVEFORM & GATE
RTS

; *** GENERATE A RANDOM NUMBER ***
RANDOMIZE = *
LDA $D41B
RANDLOOP = *
CMP $D41B
BEQ RANDLOOP
LDA $D41B
RTS

; *** SET UP FRAME DELAY ***
SETDELAY = *
LDA 162
CLC
ADC #5
STA JIFFYCLOCK
RTS

; *** WAIT FOR JIFFY CLOCK ***
WAITDELAY = *
LDA 162
CMP JIFFYCLOCK
BNE WAITDELAY
RTS

; *** RESTORE SCREEN ***
RESTORESCR = *
LDA #0
STA 53269 ;DISABLE SPRITES
LDA #4
STA 648 ;RESTORE SCREEN LOCATION
LDA #199
STA 56576 ;BANK 0
LDA 53272
AND #247
STA 53272 ;VIC OFFSET 0K
LDA 53265
AND #223
STA 53265 ;SWITCH TO TEXT MODE
JSR SHOWCREDITS
LDA #1 ;WHITE
STA 53280 ;BORDER COLOR
STA 53281 ;BACKGROUND COLOR
LDA #147
JSR $FFD2 ;CLEAR SCREEN
LDA #144
JSR $FFD2 ;SET TEXT COLOR TO BLUE
RTS

; *** SCROLL MESSAGES VERTICALLY ***

SHOWCREDITS = *
; *** INITIALIZATION ***

LDA #0 ;BLACK
STA 53281 ;BACKGROUND COLOR
STA 53280 ;BORDER COLOR
LDA #1 ;WHITE
STA 646 ;TEXT COLOR
LDA #147
JSR $FFD2 ;CLEAR SCREEN
LDA 53265
AND #247
STA 53265 ;24 ROW MODE
LDX #1
STX COUNTDOWN ;# LINES TO PAUSE
DEX
STX DEMOEND ;FLAG FOR END OF DEMO

; *** MAIN PROGRAM LOOP ***

M1 LDY #7 ; VERTICAL OFFSET IN BITS
M2 JSR SETDELY ;SET TIME DELAY
STY TEMP2
JSR RETRACE ;WAIT FOR RETRACE
LDA 53265
AND #240
ORA TEMP2
STA 53265 ;SHIFT 1 BIT VERTICALLY
JSR WAITDELY ;WAIT FOR TIME SLOT EXPIRATION
DEY
CPY #$FF ;SHIFT VALUES ARE 7-0
BNE M2 ;NEXT VERTICAL BIT SHIFT
JSR SCROLL ;SCROLL ONE WHOLE LINE
DEC COUNTDOWN ;LINES BETWEEN MESSAGES
BNE M3
JSR SHOWMSG ;SHOW MESSAGE
M3 LDA #1
CMP DEMOEND
BEQ M4
JSR $FFE4 ;CHECK FOR KEYPRESS
BEQ M1 ;REPEAT IF NONE PRESSED
M4 JSR PRISCR
LDA #27
STA 53265 ;25 ROW MODE
LDA #147
JSR $FFD2 ;CLEAR SCREEN
RTS

; *** SCROLL ONE WHOLE LINE ***

SCROLL = *

; SELECT SCREEN BUFFER

SCR0 LDA #4
CMP 648
BNE SCR0A
JSR SCR1
JMP SCR0B
SCR0A JSR SCR2
SCR0B RTS

; COPY TO ALTERNATE SCREEN

SCR1 = *

LDY #0
LOOP1 = *
LDA $428,Y
STA $2000,Y
LDA $528,Y
STA $2100,Y
LDA $628,Y
STA $2200,Y
LDA $728,Y
STA $2300,Y
INY
BNE LOOP1
LDA #32
LDY #0
LOOP2 = *
STA $23C0,Y
INY
CPY #40
BNE LOOP2

; SELECT ALTERNATE SCREEN

JSR RETRACE
SEI
LDA #32
STA 648
LDA #$85
STA 53272
CLI
RTS

; COPY TO PRIMARY SCREEN

SCR2 = *

LDY #0
LOOP3 = *
LDA $2028,Y
STA $400,Y
LDA $2128,Y
STA $500,Y
LDA $2228,Y
STA $600,Y
LDA $2328,Y
STA $700,Y
INY
BNE LOOP3
LDA #32
LDY #0
LOOP4 = *
STA $7C0,Y
INY
CPY #40
BNE LOOP4

; SELECT PRIMARY SCREEN

PRISCR = *
JSR RETRACE
SEI
LDA #4
STA 648
LDA #$15
STA 53272
CLI
RTS

; *** WAIT FOR VERTICAL RETRACE ***

RETRACE = *
LDA $D011
BPL RETRACE
RTS

; *** SET UP TIME DELAY ***

SETDELY = *
LDA 162
CLC
ADC #2
STA JIFFYCLK
RTS

; *** WAIT FOR TIME DELAY ***

WAITDELY = *
LDA 162
CMP JIFFYCLK
BNE WAITDELY
RTS

; *** SHOW MESSAGE ***

SHOWMSG = *
LDA 53265
AND #240
ORA #7
STA 53265
TXA
PHA
LDX #0
C0 = *
LDA LASTROW,X
BEQ C0END
JSR $FFD2
INX
JMP C0
C0END = *
PLA
TAX

C1 = *
LDA CREDITS,X
BEQ C2
CMP #92
BEQ C3
JSR $FFD2
INX
JMP C1
C2 INX
LDA CREDITS,X
STA COUNTDOWN
INX
RTS
C3 LDA #1
STA DEMOEND
RTS

; *** SCROLL PRG VARIABLES ***
JIFFYCLK *=*+1
TEMP2 *=*+1
DEMOEND *=*+1
COUNTDOWN !BYTE 1

LASTROW = *
!BYTE 19,17,17,17,17,17,17,17
!BYTE 17,17,17,17,17,17,17,17
!BYTE 17,17,17,17,17,17,17,17
!BYTE 17,0

CREDITS = *
!TEXT "                 K DEMO"
!BYTE 0,2
!TEXT "           BY KEVIN KRAUSNICK"
!BYTE 0,2
!TEXT "                  2007"
!BYTE 0,6
!TEXT "               GREETZ TO..."
!BYTE 0,3
!TEXT "                  KERRY"
!BYTE 0,2
!TEXT "                  YESI"
!BYTE 0,2
!TEXT "                  AMAR"
!BYTE 0,2
!TEXT "                  DAVID"
!BYTE 0,28,92

; *** BITMAP BACKGROUND PATTERNS
PATTERNS = *

;BRICKS
!BYTE $FE,$FE,$FE,0,$EF,$EF,$EF,0

;DIAGONAL BRICKS (ORIGINAL)
!BYTE $8A,$15,$0A,$44,$A0,$51,$A2,$45

;DIAGONAL BRICKS (FILLED)
!BYTE $8F,$1F,$0E,$44,$E0,$F1,$E3,$C7

;KEY
!BYTE 1,$FD,5,$75,$45,$5D,$41,$7F

;CIRCUITS
!BYTE $AD,$D6,$7B,$BD,$6B,$D6,$BD,$7B

;BUTTONS
!BYTE $55,$82,$39,$B8,$39,$80,$41,$AA

;Scales
!BYTE $1E,$D5,$DA,$6D,$AA,$67,$C1,8

;Stone
!BYTE $51,$B2,$10,0,$F7,$B2,$51,$B2

;Thatches
!BYTE 7,$8B,$DD,$B8,$70,$E8,$DD,$8E

;LINES
!BYTE $AA,$AA,$AA,$AA,$AA,$AA,$AA,$AA

;MAC
!BYTE $AA,$55,$AA,$55,$AA,$55,$AA,$55

;Waffle
!BYTE $B2,$65,$F7,$AA,$10,$65,$B2,$65

;FIELD EFFECT
!BYTE $C9,$A6,$59,$65,$9A,$6A,$95,$56

;DIZZY
!BYTE $C1,$F8,$1E,$F8,$C1,$8F,$3C,$8F

;ROUNDER
!BYTE $28,$6C,$D7,$28,$D7,$6B,$2A,$28

; *** PROGRAM VARIABLES ***
COLORS !BYTE 2,4,6,9
BDIRECTION !BYTE 0
RFRAME !BYTE 0
FFRAME !BYTE 0
BFRAME !BYTE 0
JIFFYCLOCK *=*+1
SPRITENO *=*+1
PATSTART *=*+1
PATEND *=*+1
TEMP1 *=*+1

* = $8800 ; 34816
!src "kdemo_dat.asm"